;;;; This file is part of Webgal
;;;; Copyright (C) 2004 Sergey Poznyakoff
;;;;
;;;; Webgal is free software; you can redistribute it and/or modify
;;;; it under the terms of the GNU General Public License as published by
;;;; the Free Software Foundation; either version 2 of the License, or
;;;; (at your option) any later version.
;;;;
;;;; Webgal is distributed in the hope that it will be useful,
;;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;;; GNU General Public License for more details.
;;;;
;;;; You should have received a copy of the GNU General Public License
;;;; along with Webgal; if not, write to the Free Software
;;;; Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
;;;;

;;; Given current MODE, original WIDTH and HEIGHT of the image, desired
;;; XSIZE and YSIZE, compute new dimensions of the image.
;;; Returns: (list (cons WIDTH HEIGHT) (cons X-OFF Y-OFF))
(define (compute-dimensions mode width height xsize ysize)
  (cond
   ((= mode 0)
    ;; Use characteristic size
    (let ((ratio (cond
		  ((> height width)
		   (/ xsize height))
		  (t
		   (/ xsize width)))))
      (list (cons (* width ratio) (* height ratio))
	    (cons 0 0))))

   ((= mode 1)
    ;; width and height are the same
    (let ((size xsize)
	  (new-height xsize)
	  (new-width xsize)
	  (offx 0)
	  (offy 0))
      (cond
       ((> height width)
	(set! offx (/ (- size (* size (/ width height))) 2))
	(set! new-width (* size (/ width height)) size))
       (t
	(set! offy (/ (- size (* size (/ height width))) 2))
	(set! new-height (* size (/ height width)))))
      (list (cons new-width new-height) (cons offx offy))))
   
   ((= mode 2)
    ;; Use specified height/width
    (list (cons xsize ysize) (cons 0 0)))))

;;; Create preview for the image
;;; Arguments:
;;;  IMAGE       Source image
;;;  XSIZE       Desired characteristic dimension (see MODE below)
;;;  YSIZE       Desired vertical dimension (ignored unless MODE=2)
;;;  MODE        Selects algorithm for determining resulting image
;;;              size. Values: 0 - Use characteristic size
;;;                            1 - Create square image
;;;                            2 - Use both XSIZE and YSIZE
;;;  COLOR       Color to fill the unused regions of the resulting image with.
;;;              Used when MODE=1
;;;    
;;;  PROGRESSIVE Progressive JPEG image loading
;;;  QUALITY     Quality of the resulting JPEG
;;;  SMOOTHING   Smoothing to produce the JPEG 
;;;  SHARPNESS   Sharpness to produce the JPEG 
;;;  COMMENT     Comment string to be stored in the resulting JPEG
;;;  FNAME-MOD   Pattern for generating output image file name
(define (create-preview-image image 
			      xsize
			      ysize
			      mode
			      color
			      progressive
			      quality
			      smoothing
                              sharpness
			      comment
			      fname-mod)
  (let ((height (car (gimp-image-height image)))
	(width (car (gimp-image-width image)))
	(preview (car (gimp-channel-ops-duplicate image))))

    (if (not (= (car (gimp-image-base-type preview)) RGB))
	(gimp-convert-rgb preview))

    ;; Scale and resize the image
    (let ((dim (compute-dimensions mode width height xsize ysize)))

      (gimp-image-scale preview (caar dim) (cdar dim))
      (gimp-image-flatten preview)
      (gimp-layer-add-alpha (car (gimp-image-active-drawable preview)))
      (if (or (> (caadr dim) 0)
	      (> (cdadr dim) 0))
	  (gimp-image-resize preview
			     xsize xsize
			     (caadr dim) (cdadr dim)))
      (plug-in-sharpen RUN-NONINTERACTIVE preview
		       (car (gimp-image-get-active-layer preview))
		       sharpness)
      
      ;; Fill in background
      (gimp-palette-set-foreground color)
      (gimp-palette-set-background color)
      (gimp-image-flatten preview))
    
    (gimp-layer-set-name (car (gimp-image-get-active-layer preview)) "Preview")

    ;; Create new file name
    (let* ((source-name (car (gimp-image-get-filename image)))
	   (source-len (string-length source-name))
	   (slash (let ((n 0)
			(pos 0))
		    (while (< n (- source-len 1))
			   (if (substring-equal?
				(substring source-name n (+ n 1))
				"/")
			       (set! pos n))
			   (set! n (+ n 1)))
		    (if (> pos 0)
			(+ pos 1)
			pos)))
	   (pos (string-search "&" fname-mod)))

      (let ((new-name (string-append
		       (if (substring-equal?
			    (substring fname-mod 0 1)
			    "/")
			   ""
			   (substring source-name 0 slash))
		       (substring fname-mod 0 pos)
		       (substring source-name
				  slash
				  (-
				   (string-length source-name)
				   (cond
				    ((substring-equal?
				      (substring
				       source-name
				       (- (string-length source-name) 4))
				      ".jpg")
				     4)
				    ((substring-equal?
				      (substring
				       source-name
				       (- (string-length source-name) 5))
				      ".jpeg")
				     5)
				    (t
				     0))))
		       (substring fname-mod (+ 1 pos))
		       ".jpg")))
	(file-jpeg-save
	 1
	 preview
	 (car (gimp-image-active-drawable preview))
	 new-name
	 new-name
	 quality
	 smoothing
	 1
	 progressive
	 comment
	 0
	 1
	 0
	 2)
	
	(gimp-image-set-filename preview new-name)))
    
    (gimp-image-clean-all preview)
    (gimp-display-new preview)))


(define (scripts-fu-create-preview image
				   drawable
				   xsize
				   ysize
				   mode
				   color
				   progressive
				   quality
				   smoothing
                                   sharpness
				   comment
				   fname-mod)
  (create-preview-image image xsize ysize mode color progressive
			quality smoothing sharpness comment fname-mod))

(define (scripts-fu-create-preview-batch xsize
					 ysize
					 mode
					 color
					 progressive
					 quality
					 smoothing
                                         sharpness
					 comment
					 fname-mod)
  (let ((count (car (gimp-image-list)))
	(img-list (cadr (gimp-image-list)))
	(n 0))

    (gimp-palette-set-foreground color)
    (gimp-palette-set-background color)

    (while (< n count)
	   (create-preview-image (aref img-list n) xsize ysize mode color
				 progressive quality smoothing sharpness
				 comment fname-mod)
	   (set! n (+ n 1)))))
    

(script-fu-register
 "scripts-fu-create-preview"
 "<Image>/Script-Fu/Webgal/Create Preview Image..."
 "Creates a preview version of the specified image. Output file name is obtained from 'Modify Filename' string, using following rules: 1) Any character in this string, except & is copied to the output file name verbatim; 2) The first occurrence of & is replaced with the source file name; 3) The source file directory is prepended to the output file name unless 'Modify Filename' begins with a slash.\
If mode is set to 'Characteristic Size' the largest dimension of the image is scaled to the value of 'New Width' and the other dimension is scaled proportionally.\
If the mode is 'Square' both dimensions of the new image are set to the value of 'New Width' and the image is scaled and resized to fit within the resulting square.\
If the mode is 'Width/Height' the specified values of width and height are used even if that would result in distorting the image."
 "Sergey Poznyakoff and Wojciech Polak"
 "Sergey Poznyakoff and Wojciech Polak"
 "October 12, 2004"
 "RGB*"

 SF-IMAGE       _"Source Image"         0
 SF-DRAWABLE    _"Our drawable"         0
 SF-ADJUSTMENT  _"New Width"            '(128 1 1024 1 10 0 1)
 SF-ADJUSTMENT  _"Height"               '(128 1 1024 1 10 0 1)
 SF-OPTION      _"Mode"                 '(_"Characteristic Size"
					   _"Square" 
					   _"Width/Height")  
 SF-COLOR       _"Background Color"     '(255 255 255)
 SF-TOGGLE      _"Progressive"          '0
 SF-ADJUSTMENT  _"JPEG Quality"         '(0.8 0.00 1.00 .01 .01 2 0)
 SF-ADJUSTMENT  _"JPEG Smoothing"       '(0.00 0.00 1.00 .01 .1 2 0)
 SF-ADJUSTMENT  _"Sharpness"            '(40 1 99 1 10 0 0) 
 SF-STRING      _"Image Comment"        "Created with the Gimp!"
 SF-STRING      _"Modify Filename"      "preview/&")


(script-fu-register
 "scripts-fu-create-preview-batch"
 "<Toolbox>/Xtns/Script-Fu/Webgal/Create Previews"
 "For all currently open images in GIMP creates corresponding preview versions. The names of output files are obtained from 'Modify Filename' string using following rules: 1) Any character in this string, except & is copied to the output file name verbatim; 2) The first occurrence of & is replaced with the source file name; 3) The source file directory is prepended to the output file name unless 'Modify Filename' begins with a slash.\
If the mode is set to 'Characteristic Size' the largest dimension of the image is scaled to the value of 'New Width' and the other dimension is scaled proportionally.\
If the mode is 'Square' both dimensions of the new image are set to the value of 'New Width' and the image is scaled and resized to fit within the resulting square.\
If mode is 'Width/Height' the specified values of width and height are used even if that would result in distorting the image."
 "Sergey Poznyakoff and Wojciech Polak"
 "Sergey Poznyakoff and Wojciech Polak"
 "October 12, 2004"
 "RGB*"
 SF-ADJUSTMENT  _"New Width"            '(128 1 1024 1 10 0 1)
 SF-ADJUSTMENT  _"Height"               '(128 1 1024 1 10 0 1)
 SF-OPTION      _"Mode"                 '(_"Characteristic Size"
					   _"Square" 
					   _"Width/Height") 
 SF-COLOR       _"Background Color"     '(255 255 255)
 SF-TOGGLE      _"Progressive"          '0
 SF-ADJUSTMENT  _"JPEG Quality"         '(0.8 0.00 1.00 .01 .01 2 0)
 SF-ADJUSTMENT  _"JPEG Smoothing"       '(0.00 0.00 1.00 .01 .1 2 0)
 SF-ADJUSTMENT  _"Sharpness"            '(40 1 99 1 10 0 0) 
 SF-STRING      _"Image Comment"        "Created with the Gimp!"
 SF-STRING      _"Modify Filename"      "preview/&")

